/* $LAAS: viam_private.h 2013/09/25 15:34:16 mallet $
 *
 * Copyright (c) 2007-2008,2010,2013 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *                                      Anthony Mallet on Tue Jan 23 2007
 */
#ifndef H_VIAM_PRIVATE
#define H_VIAM_PRIVATE

#include "viam/viamlib.h"
#include "viam/viam_acq_driver.h"


/* --- Forward decls ------------------------------------------------------- */

typedef struct viam_bankcalib_t *viam_bankcalib_t;
typedef struct viam_cameracalib_t *viam_cameracalib_t;


/* --- Library generic routines -------------------------------------------- */

viam_camera_t	viam_config_cameras(viam_handle_t vh);
void		viam_config_setcameras(viam_handle_t vh, viam_camera_t camera);

viam_bank_t	viam_config_banks(viam_handle_t vh);
void		viam_config_setbanks(viam_handle_t vh, viam_bank_t bank);

viam_image_ref	viam_config_images(viam_handle_t vh);
void		viam_config_setimages(viam_handle_t vh, viam_image_ref image);

viam_filter_t	viam_config_filters(viam_handle_t vh);
void		viam_config_setfilters(viam_handle_t vh, viam_filter_t filter);


/* --- Internal image structure -------------------------------------------- */

#ifndef VIAM_OPENCV_SUPPORT

/* Provide our simple replacement for the missing opencv structures and
 * functions when configured without opencv support. Note that the IplImage
 * provided here is not compatible with opencv's one. */

typedef struct _IplImage {
  int  nChannels;		/**< 1 or 3 channels */
  int  depth;			/**< pixel depth in bits */
  int  width;			/**< image width in pixels */
  int  height;			/**< image height in pixels */
  int  imageSize;		/**< image data size in bytes */
  unsigned char *imageData;	/**< pointer to aligned image data */
  int  widthStep;		/**< size of aligned image row in bytes */
} IplImage;

typedef struct CvSize {
  int width;
  int height;
} CvSize;

#define IPL_DEPTH_8U	8
#define IPL_DEPTH_16U	16
#define IPL_DEPTH_32F	32

#define IPL_ORIGIN_TL	0

#define CV_AUTOSTEP	0x7fffffff

#define CV_LOAD_IMAGE_UNCHANGED  -1

CvSize		cvSize(int width, int height);
IplImage *	cvCreateImage(CvSize size, int depth, int channels);
IplImage *	cvCreateImageHeader(CvSize size, int depth, int channels);
IplImage *	cvInitImageHeader(IplImage *image, CvSize size, int depth,
			int channels, int origin, int align);
void		cvSetData(struct _IplImage *image, void *data, int step);
int		cvSaveImage(const char *filename, const struct _IplImage *image);
IplImage *	cvLoadImage(const char* filename, int iscolor);
void		cvReleaseImage(IplImage **image);

#endif /* !VIAM_OPENCV_SUPPORT */


/* --- Bank and camera configuration --------------------------------------- */

viam_bank_t		viam_bank_next(viam_bank_t camera);
viam_bankcalib_t	viam_bank_caldata(viam_bank_t bank);
void			viam_bank_setcaldata(viam_bank_t bank,
				viam_bankcalib_t c);
void			viam_bank_setconfigured(viam_bank_t bank,
				int configured);
void *			viam_bank_hwdata(viam_bank_t bank);
void			viam_bank_sethwdata(viam_bank_t bank, void *hwdata);

viam_camera_t		viam_camera_next(viam_camera_t camera);
viam_acqdev_obj		viam_camera_hwdev(viam_camera_t camera);
void			viam_camera_sethwdev(viam_camera_t camera,
				viam_acqdev_obj hwdev);
int			viam_camera_setup(viam_camera_t camera);
void			viam_camera_setsetup(viam_camera_t camera,
				int setup);
struct _IplImage *	viam_camera_raw(viam_camera_t camera);
/* void *			viam_camera_hwdata(viam_camera_t camera); */
/* void			viam_camera_sethwdata(viam_camera_t camera, */
/* 				void *hwdata); */

viam_image_ref		viam_image_create(viam_handle_t vh, const char *name,
				viam_camera_t camera);
viam_image_ref		viam_image_next(viam_image_ref image);
viam_filter_t		viam_image_flist(const viam_image_ref image);
void			viam_image_setflist(viam_image_ref image,
				viam_filter_t flist);
viam_cameracalib_t	viam_image_caldata(const viam_image_ref image);
void			viam_image_setcaldata(viam_image_ref image,
				viam_cameracalib_t c);

void *			viam_swap(unsigned int bank);
void			viam_setswap(unsigned int bank, size_t size);


/* --- Image filters ------------------------------------------------- */

int	viam_filter_inithw(viam_handle_t vh, viam_image_ref image);
int	viam_filter_image(viam_handle_t vh, viam_image_ref camera,
		struct _IplImage *final);
int	viam_filter_configure(viam_handle_t vh, viam_image_ref camera,
		viam_filter_t f);
int	viam_filter_geo(viam_handle_t vh, viam_filter_t f,
		viam_geometry_t *geom);
int	viam_filter_queryswfeature(viam_handle_t vh, viam_filtertype_t filter,
		unsigned int *methods, unsigned int *aumodes);

int	viam_format_geosetup(viam_handle_t vh, viam_image_ref image,
		viam_filterargs_obj args, viam_geometry_t *geom, void **data);
int	viam_cvtformat(viam_handle_t vh, viam_image_ref image,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_obj args, void *data);

int	viam_geo_geosetup(viam_handle_t vh, viam_image_ref camera,
		viam_filterargs_obj args, viam_geometry_t *geom, void **data);
int	viam_geo_scale(viam_handle_t vh, viam_image_ref camera,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_obj args, void *data);
#ifdef VIAM_OPENCV_SUPPORT
int	viam_geo_subsample(viam_handle_t vh, viam_image_ref camera,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_obj args, void *data);
int	viam_geo_remap(viam_handle_t vh, viam_image_ref camera,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_obj args, void *data);
int	viam_geo_remap_nn(viam_handle_t vh, viam_image_ref camera,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_obj args, void *data);
int	viam_geo_distortsetup(viam_handle_t vh, viam_image_ref camera,
		viam_filterargs_obj args, viam_geometry_t *geom, void **data);
int	viam_geo_rectifysetup(viam_handle_t vh, viam_image_ref camera,
		viam_filterargs_obj args, viam_geometry_t *geom, void **data);
#endif /* VIAM_OPENCV_SUPPORT */

int	viam_luminance_geosetup(viam_handle_t vh, viam_image_ref image,
		viam_filterargs_obj args, viam_geometry_t *geom, void **data);

int	viam_color_geosetup(viam_handle_t vh, viam_image_ref camera,
		viam_filterargs_obj args, viam_geometry_t *geom, void **data);
int	viam_awb(viam_handle_t vh, viam_image_ref camera,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_obj args, void *data);
int	viam_awb_auto(viam_handle_t vh, viam_image_ref camera,
		const struct _IplImage *source, viam_filterautomode_t *aumode,
		viam_filterargs_obj args, void *data);

#ifdef VIAM_OPENCV_SUPPORT
int	viam_improc_log(viam_handle_t vh, viam_image_ref camera,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_obj args, void *data);
#endif /* VIAM_OPENCV_SUPPORT */


/* --- Hardware bus access ------------------------------------------- */

/* Generic */
const char *	viam_hwdev_name(viam_acqdev_obj hwdev);
viam_acqdev_obj viam_hardware_device(viam_handle_t vh, const char *id,
			char **uid);
int		viam_sethwfeaturemode(viam_handle_t vh, viam_camera_t camera,
			viam_filtertype_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode);
int		viam_sethwfeaturevalue(viam_handle_t vh, viam_camera_t camera,
			viam_filtertype_t filter,
			const viam_filterargs_obj args);


/* --- Image calibration --------------------------------------------- */

void	viam_initrectifmap(const double a[9], const double distort[4],
		const double rectif[9], float *mapx, float *mapy,
		int width, int height);

#endif /* H_VIAM_PRIVATE */
